# es-beautifier

[![Build Status](https://travis-ci.org/dai-shi/es-beautifier.svg?branch=master)](https://travis-ci.org/dai-shi/es-beautifier)
[![npm version](https://badge.fury.io/js/es-beautifier.svg)](https://badge.fury.io/js/es-beautifier)

ECMAScript beautifier based on [eslint](http://eslint.org/)

WIP

## Motivation

JS Beautifier is so great that it can reduce formatting issues, however,
there are two major issues: a) customization and b) ES.next.

a) If you like the result of code generated by JS Beautifier,
it's just fine, but if you can't manage with its configuration options,
you are out of luck.

b) If you are using a new ECMAScript feature or features that are not
even standardized, you are also out of luck, because the parser
does not support the features. Among such is JSX.

ESLint is a linting tool widely used, which adopts a pluggable
architecture so that it's highly customizable.
It also has an ability to automatically fix problems.
There's lots of plugins developed, forming a big ecosystem.

So, why not build a beautifying tool using eslint?

## Screencast

TODO

## Usage (CLI)

### Install

```
npm install es-beautifier -g --only=production
```

```
es-beautifier --help
```

### Example

```
es-beautifier < file.js > file-beautified.js
```

```
es-beautifier file-to-be-beautified.js
```

## Usage (vim)

### Example with NeoBundle

```
NeoBundle 'dai-shi/es-beautifier', {'rtp': 'contrib/vim', 'external_commands': 'node', 'build_commands': 'npm', 'build': {'others': 'npm install --only=production'}}
autocmd FileType javascript nnoremap <buffer> <Leader>e :call EsBeautifier()<cr>
autocmd FileType javascript vnoremap <buffer> <Leader>e :call RangeEsBeautifier()<cr>
```

## Usage (atom)

Help wanted.

## Usage (eslint-plugin)

You can customize it just like a normal eslint plugin.

### Install

In your project directory:

```
npm install eslint eslint-plugin-es-beautifier --save-dev
```

### Example

Add the following to your `.eslintrc` or `eslintConfig` in package.json.

```
{
  "plugins": [
    "es-beautifier"
  ],
  "extends": [
    "plugin:es-beautifier/standard"
  ]
}
```

Add the following `scripts` in package.json.

```
{
  "scripts": {
    "beautify": "eslint --fix ."
  }
}
```

Run:

```
npm run beautify
```

## Similar projects

There are several tools that do smiliar to es-beautifier.

- [js-beautify](https://github.com/beautify-web/js-beautify)
- [uglify-js](https://github.com/mishoo/UglifyJS2)
- [esformatter](https://github.com/millermedeiros/esformatter)
- [prettydiff](https://github.com/prettydiff/prettydiff)

To see the comparison:

```
git clone https://github.com/dai-shi/es-beautifier.git
cd es-beautifier
npm install
npm run examples
```
